% Singular Spectrum Analysis
clear all
hold off

% Choice of the Parameter and the dates
% The parameter can be chosen from this list:
% 'p1_0','p1_1','p2_0','p2_1','p3_0','p3_1',
% 'fp1_0','fp1_1','fp2_0','fp2_1','fp3_0','fp3_1',
% 'psi_0', 'psi_1'
par='psi_1';
if par(end)=='0'
    sens='NS';
elseif par(end)=='1'
    sens='EW';
end
month_ini=[2013,03,01];
month_fin=[2018,02,28];
row_ini=datenum(month_ini)-datenum(2013,03,01)+1;
row_fin=datenum(month_fin)-datenum(2013,03,01)+1;
Nd=row_fin-row_ini;

% Loading of the Data
[dat_r,txt,raw] = xlsread(par) ;
dat=dat_r(row_ini:row_fin,:);

% Average for each day (rows) and for each hour of the day (columns)
sum_d=nansum(dat,2);
count_d=sum(~isnan(dat),2);
mean_d=sum_d./count_d;
sum_h=nansum(dat,1);
count_h=sum(~isnan(dat),1);
mean_h=sum_h./count_h;
mean_tot=nanmean(mean_d);

% Filling of the day average vector for the days that are empty of data
mean_d_fil=mean_d;
for i=1:Nd
    if(isnan(mean_d_fil(i)))
        if i>345 && i<Nd-384
            if any(isnan([mean_d(max([1,i-385]):i-345); mean_d(i+345:min([i+385,Nd]))]))
                mean_vert=nanmean([mean_d(max([1,i-385]):i-345); mean_d(i+345:min([i+385,Nd]))]);
            else
                mean_vert=mean_tot;
            end
        end
        if i>20 && i<Nd-19
            if any(isnan([mean_d(max([1,i-20]):i-1); mean_d(i+1:min([i+20,Nd]))]))
                mean_hor=nanmean([mean_d(max([1,i-20]):i-1); mean_d(i+1:min([i+20,Nd]))]);
            else
                mean_hor=mean_tot;
            end
        end
        mean_d_fil(i)=mean_vert*mean_hor/mean_tot;
    end
end

% UNCOMMENT TO PLOT THE FILLING OF THE DATA
% plot(1:length(mean_d),mean_d_fil,'r')
% hold on
% plot(1:length(mean_d),mean_d,'b')
% xlim([0,length(mean_d)]);
% set(gcf,'PaperSize',[18,14]);
% xlabel('Time (days)');
% ylabel([par(1:end-2), ' (', sens,')']);
% suptitle({['Filling of the blank data ', par(1:end-2), ' (', sens, ')']})
% legend('Filled Data','Original Data with Blanks')
% print -dpdf -fillpage -painters Filling_2.pdf

% Application of the SSA. Construction of the matrix X
N=length(mean_d_fil);
L=2*365;
X=zeros(L,N-L+1);
for i=1:L
    for j=1:N-L+1
        X(i,j)=mean_d_fil(i+j-1);
    end
end

% Singular Values (Lambda) and Singular Vectors (U, V)
A=X*transpose(X);
[Up,Dp] = eig(A);
[minus_Lambda,ind] = sort(-diag(Dp));
Lambda=-minus_Lambda;
L_nor=Lambda/sum(Lambda);
U = Up(:,ind);
V=transpose(X)*U;
for i=1:length(Lambda)
    V(:,i)=V(:,i)/sqrt(Lambda(i));
end

% Reconstruction with Xp using the first 'd' principal components
Xr=zeros(L,N-L+1);
d=300;
for i=1:d
    eval(['Xp' int2str(i) '=sqrt(Lambda(' int2str(i) '))*U(:,' int2str(i) ')*transpose(V(:,' int2str(i) '));']);
    eval(['Xr=Xr+Xp' int2str(i) ';']);
end

% UNCOMMENT TO PLOT THE RECONSTRUCTION OF THE SIGNAL
% set(gcf,'PaperSize',[18,14]);
% plot(mean_d_fil(1:730));
% hold on;
% plot(Xr(1,(1:730)));
% xlabel('Time (days)');
% ylabel([par(1:end-2), ' (', sens,')']);
% xlim([0,730]);
% suptitle({['Reconstruction of the original signal of ', par(1:end-2), ' (', sens, ')'],['using the first ', int2str(d) ,' principal components']})
% legend('Original Filled Data','Reconstruction of the Signal')
% print -dpdf -fillpage -painters Recons.pdf

