% Singular Spectrum Analysis
% clear all
hold off

% Choice of the Parameter and the dates
% The parameter can be chosen from this list:
% 'p1_0','p1_1','p2_0','p2_1','p3_0','p3_1',
% 'fp1_0','fp1_1','fp2_0','fp2_1','fp3_0','fp3_1',
% 'psi_0', 'psi_1'
par='p1_1';
if par(end)=='0'
    sens='NS';
elseif par(end)=='1'
    sens='EW';
end
month_ini=[2013,03,01];
month_fin=[2018,02,28];
row_ini=datenum(month_ini)-datenum(2013,03,01)+1;
row_fin=datenum(month_fin)-datenum(2013,03,01)+1;
Nd=row_fin-row_ini;

% Loading of the Data
[dat_r,txt,raw] = xlsread(par) ;
dat=dat_r(row_ini:row_fin,:);

% Average for each day (rows) and for each hour of the day (columns)
sum_d=nansum(dat,2);
count_d=sum(~isnan(dat),2);
mean_d=sum_d./count_d;
sum_h=nansum(dat,1);
count_h=sum(~isnan(dat),1);
mean_h=sum_h./count_h;
mean_tot=nanmean(mean_d);

% Filling of the day average vector for the days that are empty of data
mean_d_fil=mean_d;
for i=1:Nd
    if(isnan(mean_d_fil(i)))
        if i>345 && i<Nd-384
            if any(isnan([mean_d(max([1,i-385]):i-345); mean_d(i+345:min([i+385,Nd]))]))
                mean_vert=nanmean([mean_d(max([1,i-385]):i-345); mean_d(i+345:min([i+385,Nd]))]);
            else
                mean_vert=mean_tot;
            end
        end
        if i>20 && i<Nd-19
            if any(isnan([mean_d(max([1,i-20]):i-1); mean_d(i+1:min([i+20,Nd]))]))
                mean_hor=nanmean([mean_d(max([1,i-20]):i-1); mean_d(i+1:min([i+20,Nd]))]);
            else
                mean_hor=mean_tot;
            end
        end
        mean_d_fil(i)=mean_vert*mean_hor/mean_tot;
    end
end

% Application of the SSA and construction of X matrix
N=length(mean_d_fil);
L=2*365;
X=zeros(L,N-L+1);
for i=1:L
    for j=1:N-L+1
        X(i,j)=mean_d_fil(i+j-1);
    end
end

% SINGULAR VALUES (Lambdas) and Singular Vectors (U, V)
A=X*transpose(X);
[Up,Dp] = eig(A);
[minus_Lambda,ind] = sort(-diag(Dp));
Lambda=-minus_Lambda;
L_nor=Lambda/sum(Lambda);
reset(gca)
reset(gcf)
hold off;
ax=subplot(1,1,1);
plot(ax,1:11,Lambda(1:11))
set(gca,'yscale','log')
suptitle(['Principal components eigenvalues for ', par(1:end-2), ' (', sens, ') , with 1 sample per day'])
xlabel('n','FontSize',13)
ylabel(['Relative intensity (log scale)'],'FontSize',13)
set(gcf,...
    'PaperSize',[20,18]);
print -dpdf -fillpage -painters Eigenvalues.pdf
hold off;
reset(gca)
reset(gcf)
U = Up(:,ind);
V=transpose(X)*U;
for i=1:length(Lambda)
    V(:,i)=V(:,i)/sqrt(Lambda(i));
end

% Reconstruction with Xp
Xr=zeros(L,N-L+1);
d=100;
for i=1:d
    eval(['Xp' int2str(i) '=sqrt(Lambda(' int2str(i) '))*U(:,' int2str(i) ')*transpose(V(:,' int2str(i) '));']);
    eval(['Xr=Xr+Xp' int2str(i) ';']);
end

% Hankelization
for l=1:d
    l
    eval(['Y' int2str(l) '=zeros(1,N);']);
    for k=1:N
        max_i=min([k,L]);
        min_i=max([k+L-N,1]);
        min_j=max([1,k-L+1]);
        max_j=min([k,N-L+1]);
        indx_i=max_i:-1:min_i;
        indx_j=min_j:max_j;
        indx=indx_i+L*(indx_j-1);
        eval(['sum_k=mean(Xp' int2str(l) '(indx));']);
        eval(['Y' int2str(l) '(k)=sum_k;']);
    end
end

% Reconstruction of the original signal with Y
Y=zeros(1,N);
for i=1:d
    eval(['Y=Y+Y' int2str(i) ';']);
end

% CORRELATIONS AND SEPARABILITY
rho=zeros(d,d);
w=zeros(1,N);
for k=1:N
    w(k)=min([k,L,N-k]);
end
for i=1:d
    for j=1:d
        eval(['norm1=sqrt(sum(w.*Y' int2str(i) '.*Y' int2str(i) '));']);
        eval(['norm2=sqrt(sum(w.*Y' int2str(j) '.*Y' int2str(j) '));']);
        eval(['rho(i,j)=sum(w.*Y' int2str(i) '.*Y' int2str(j) ')/(norm1*norm2);']);
    end
end
hold off
s=imagesc(1:14,1:14,rho(1:14,1:14));
colormap summer;
colorbar
suptitle({['Correlation among principal components'],['of ', par(1:end-2), ' (', sens, ') , taking 1 sample per day']})
xlabel('Principal component n','FontSize',13)
ylabel(['Principal component n'],'FontSize',13)
set(gcf,...
    'PaperSize',[18,16]);
print -dpdf -fillpage -painters Correlation.pdf
hold off

for i=1:d
    eval(['FT' int2str(i) '=abs(fft(U(:,' int2str(i) ')));']);
end
f=(0:L/2-1)/L;
fprima=1./f;

% We have to decide the eigenvalues corresponding to the same kind of variation
if strcmp(par,'p1_0')
    C={[1],[2,3],[4,5],[6],[7,8],[9,10],[11,12],[13,14]};
elseif strcmp(par,'p1_1')
    C={[1],[2,3],[4],[5],[6,7],[8,9],[10,11],[12,13,14]};
elseif strcmp(par,'p2_0')
    C={[1],[2,3],[4,5],[6,7],[8,9],[10],[11],[12,13]};
elseif strcmp(par,'p2_1')
    C={[1],[2,3],[4],[5,6],[7],[8,9],[10,11],[12,13]};
elseif strcmp(par,'p3_0')
    C={[1],[2,3],[4,5],[6,7],[8,9],[10],[11,12],[13,14]};
elseif strcmp(par,'p3_1')
    C={[1],[2,3],[4,5],[6],[7],[8,9],[10,11],[12,13]};
elseif strcmp(par,'fp1_0')
    C={[1],[2],[3,4,5],[6],[7],[8],[9,10],[11,12]};
elseif strcmp(par,'fp1_1')
    C={[1],[2,3,4],[5],[6,7],[8,9,10],[11],[12,13],[14]};
elseif strcmp(par,'fp2_0')
    C={[1],[2,3],[4,5,6],[7,8],[9,10],[11],[12,13],[14]};
elseif strcmp(par,'fp2_1')
    C={[1],[2,3],[4,5],[6,7],[8,9],[10,11],[12],[13,14]};
elseif strcmp(par,'fp3_0')
    C={[1],[2,3],[4,5],[6,7,8],[9,10],[11],[12,13],[14]};
elseif strcmp(par,'fp3_1')
    C={[1],[2,3],[4,5],[6,7],[8,9],[10],[11,12,13],[14]};
elseif strcmp(par,'psi_0')
    C={[1],[2,3],[4,5],[6,7],[8,9],[10],[11,12],[13]};  
elseif strcmp(par,'psi_1')
    C={[1],[2,3],[4,5],[6,7],[8,9],[10],[11,12],[13,14]};    
end
%C={[1],[2,3],[4,5],[6],[7,8,11,12],[9,10],[13],[14]};

% We add the eigenvectors which show the same kind of variation
for i=1:length(C)
    eval(['Ut' int2str(i) '=zeros(L,1);']);
    for j=1:length(C{i})
        eval(['Ut' int2str(i) '=Ut' int2str(i) '+U(:,' int2str(C{i}(j)) ');']);
    end
    eval(['FTC' int2str(i) '=abs(fft(Ut' int2str(i) '));']);
end

% Plot of the PRINCIPAL COMPONENTS - Unitary eigenvectors, Ut(:,i)
cP=0;
for i=1:max([length(C),8]) 
    cP=cP+1;   
    eval(['ax' int2str(cP) '=subplot(4,2,' int2str(cP) ');']);
    eval(['plot(ax' int2str(cP) ',1:L,Ut' int2str(cP) ');']);
    s=L_nor(C{i}(1));
    charstr=['Components ' int2str(C{i}(1))];
    for iauto=2:length(C{i})
        s=s+L_nor(C{i}(iauto));
        charstr=[charstr ', ' int2str(C{i}(iauto))];
    end
    tit=[charstr '. Contr = ' num2str(100*s) ' %'];
    eval(['title(ax' int2str(cP) ', tit);']);
    if cP>6
        xlab='Time (Days)';
        eval(['xlabel(ax' int2str(cP) ', xlab);']);
    end
end
set(gcf,...
    'PaperSize',[18,18]);
suptitle({['PRINCIPAL COMPONENTS OF ', par(1:end-2), ' (', sens, ')'],['TAKING 1 SAMPLE PER DAY']})
print -dpdf -fillpage -painters Components.pdf

% Plot of the FFT of the PRINCIPAL COMPONENTS (i.e., of the unitary eigenvectors), FTC(:,i)
cP=0;
for i=1:max([length(C),8]) 
    cP=cP+1;   
    eval(['ax' int2str(cP) '=subplot(4,2,' int2str(cP) ');']);
    eval(['plot(ax' int2str(cP) ',fprima,FTC' int2str(cP) '(1:end/2));']);
    s=L_nor(C{i}(1));
    charstr=['Components ' int2str(C{i}(1))];
    for iauto=2:length(C{i})
        s=s+L_nor(C{i}(iauto));
        charstr=[charstr ', ' int2str(C{i}(iauto))];
    end
    tit=[charstr '. Contr = ' num2str(100*s) ' %'];
    eval(['title(ax' int2str(cP) ', tit);']);
    if cP>6
        xlab='Period (Days)';
        eval(['xlabel(ax' int2str(cP) ', xlab);']);
    end
end
set(gcf,...
    'PaperSize',[18,18]);
suptitle({['FDTD OF THE PRINCIPAL COMPONENTS OF ', par(1:end-2), ' (', sens, ')'],['TAKING 1 SAMPLE PER DAY']})
print -dpdf -fillpage -painters FDFT_of_components.pdf